program pnpclnt;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  pnpshr  in '../pnpshare/pnpshr.pas',
  pnpclss in '../pnpshare/pnpclss.pas';

var
  VExit        : Boolean;
  VIdPNPClient : TIdPNPClient;
  VLine        : String;

const
  CExitA = 'EXIT';
  CExitB = 'E';

  CHelpA = 'HELP';
  CHelpB = 'H';

  CQuoteA = 'QUOTE';
  CQuoteB = 'Q';

  CSetHostA = 'SETHOST';
  CSetHostB = 'SH';

  CSetPortA = 'SETPORT';
  CSetPortB = 'SP';

  CSetTimeoutA = 'SETTIMEOUT';
  CSetTimeoutB = 'ST';

  CParameterDivider = ' ';

  CParamHelpA = '-HELP';
  CParamHelpB = '-H';
  CParamHelpC = '-?';

  CParamSetHostA = '-SETHOST';
  CParamSetHostB = '-SHOST';
  CParamSetHostC = '-SH';

  CParamSetPortA = '-SETPORT';
  CParamSetPortB = '-SPORT';
  CParamSetPortC = '-SP';

  CParamSetTimeoutA = '-SETTIMEOUT';
  CParamSetTimeoutB = '-STIMEOUT';
  CParamSetTimeoutC = '-ST';

procedure Connect;
begin
  while not(VIdPNPClient.Connected) do
  begin
    try
      VIdPNPClient.Connect(VIdPNPClient.ReadTimeout);
    except
      WriteLn('< CONNECTION FAILED');
    end;
  end;
end;

procedure ExecuteCommand(ACommand : String; out AExit : Boolean);
var
  LCommand   : String;
  LDone      : Boolean;
  LLine      : String;
  LNumber    : LongInt;
  LParameter : String;
  LQuote     : Boolean;
begin
  ACommand := AnsiUpperCase(Trim(ACommand));
  AExit    := false;

  if (Pos(CParameterDivider, ACommand) > 0) then
  begin
    LCommand   := Copy(ACommand, 1, Pred(Pos(CParameterDivider, ACommand)));
    LParameter := Copy(ACommand, Succ(Pos(CParameterDivider, ACommand)), Length(ACommand) - Pos(CParameterDivider, ACommand));
  end
  else
  begin
    LCommand   := ACommand;
    LParameter := '';
  end;

  LDone  := false;
  LQuote := false;
  if ((LCommand = CExitA) or (LCommand = CExitB)) then
  begin
    AExit := true;

    WriteLn('< STOPPING EXECUTION');

    LDone := true;
  end;
  if ((LCommand = CHelpA) or (LCommand = CHelpB)) then
  begin
    WriteLn('< SHOWING HELP');
    WriteLn('EXIT       : ' + CExitA       + ' | ' + CExitB);
    WriteLn('SHOW HELP  : ' + CHelpA       + ' | ' + CHelpB);
    WriteLn('QUOTE      : ' + CQuoteA      + ' | ' + CQuoteB      + ' <command>');
    WriteLn('SET HOST   : ' + CSetHostA    + ' | ' + CSetHostB    + ' <host>');
    WriteLn('SET PORT   : ' + CSetPortA    + ' | ' + CSetPortB    + ' <number>');
    WriteLn('SET TIMEOUT: ' + CSetTimeoutA + ' | ' + CSetTimeoutB + ' <number>');
    WriteLn('< HELP SHOWN');

    LDone := true;
  end;
  if ((LCommand = CQuoteA) or (LCommand = CQuoteB)) then
  begin
    LQuote := true;

    LDone := true;
  end;
  if ((LCommand = CSetHostA) or (LCommand = CSetHostB)) then
  begin
    WriteLn('< SETTING HOST');

    if VIdPNPClient.Connected then
      VIdPNPClient.Disconnect;

    try
      VIdPNPClient.Host := LParameter;

      WriteLn('< HOST SET');
    except
      WriteLn('< ERROR: HOST NOT SET SUCCESSFULLY');
    end;

    LDone := true;
  end;
  if ((LCommand = CSetPortA) or (LCommand = CSetPortB)) then
  begin
    WriteLn('< SETTING PORT');

    if VIdPNPClient.Connected then
      VIdPNPClient.Disconnect;

    LNumber := StrToIntDef(LParameter, 0);
    if (LNumber > 0) then
    begin
      try
        VIdPNPClient.Port := LNumber;

        WriteLn('< PORT SET');
      except
        WriteLn('< ERROR: PORT NOT SET SUCCESSFULLY');
      end;
    end
    else
      WriteLn('< ERROR: PORT NOT SET SUCCESSFULLY');

    LDone := true;
  end;
  if ((LCommand = CSetTimeoutA) or (LCommand = CSetTimeoutB)) then
  begin
    WriteLn('< SETTING TIMEOUT');

    if VIdPNPClient.Connected then
      VIdPNPClient.Disconnect;

    LNumber := StrToIntDef(LParameter, 0);
    if (LNumber > 0) then
    begin
      try
        VIdPNPClient.ReadTimeout := LNumber;

        WriteLn('< TIMEOUT SET');
      except
        WriteLn('< ERROR: TIMEOUT NOT SET SUCCESSFULLY');
      end;
    end
    else
      WriteLn('< ERROR: TIMEOUT NOT SET SUCCESSFULLY');

    LDone := true;
  end;

  if (not(LDone) or LQuote) then
  begin
    if LQuote then
      WriteLn('< QUOTING COMMAND');

    Connect;
    if LQuote then
      VIdPNPClient.WriteLn(LParameter)
    else
      VIdPNPClient.WriteLn(ACommand);

    try
      repeat
        Connect;
        LLine := VIdPNPClient.ReadLn;

        if (Length(Trim(LLine)) > 0) then
          WriteLn(LLine);
      until (Length(Trim(LLine)) <= 0);
    except
    end;

    if LQuote then
      WriteLn('< COMMAND QUOTED');
  end;
end;

procedure ExecuteParameters(out AShutdown : Boolean);
type
  TParameterEnum = (peNone, peSetHost, peSetPort, peSetTimeout);
var
  LDone      : Boolean;
  LIndex     : LongInt;
  LNumber    : LongInt;
  LParameter : String;
  LProgress  : TParameterEnum;
begin
  AShutdown := false;

  LProgress := peNone;
  for LIndex := 1 to ParamCount do
  begin
    LParameter := AnsiUpperCase(Trim(ParamStr(LIndex)));

    case LProgress of
      peSetHost :
      begin
        WriteLn('< SETTING HOST');

        try
          VIdPNPClient.Host := LParameter;

          WriteLn('< HOST SET');
        except
          WriteLn('< ERROR: HOST NOT SET SUCCESSFULLY');
        end;

        LProgress := peNone;
      end;

      peSetPort :
      begin
        WriteLn('< SETTING PORT');

        LNumber := StrToIntDef(LParameter, 0);
        if (LNumber > 0) then
        begin
          try
            VIdPNPClient.Port := LNumber;

            WriteLn('< PORT SET');
          except
            WriteLn('< ERROR: PORT NOT SET SUCCESSFULLY');
          end;
        end
        else
          WriteLn('< ERROR: PORT NOT SET SUCCESSFULLY');

        LProgress := peNone;
      end;

      peSetTimeout :
      begin
        WriteLn('< SETTING TIMEOUT');

        LNumber := StrToIntDef(LParameter, 0);
        if (LNumber > 0) then
        begin
          try
            VIdPNPClient.ReadTimeout := LNumber;

            WriteLn('< TIMEOUT SET');
          except
            WriteLn('< ERROR: TIMEOUT NOT SET SUCCESSFULLY');
          end;
        end
        else
          WriteLn('< ERROR: TIMEOUT NOT SET SUCCESSFULLY');

        LProgress := peNone;
      end;
    else
      if (LProgress = peNone) then
      begin
        LDone := false;

        if ((LParameter = CParamHelpA) or (LParameter = CParamHelpB) or (LParameter = CParamHelpC)) then
        begin
          AShutdown := true;

          WriteLn('< SHOWING HELP');
          WriteLn('SHOW HELP  : ' + CParamHelpA        + ' | ' + CParamHelpB        + ' | ' + CParamHelpC);
          WriteLn('SET HOST   : ' + CParamSetHostA     + ' | ' + CParamSetHostB     + ' | ' + CParamSetHostC    + ' <host>');
          WriteLn('SET PORT   : ' + CParamSetPortA     + ' | ' + CParamSetPortB     + ' | ' + CParamSetPortC    + ' <number>');
          WriteLn('SET TIMEOUT: ' + CParamSetTimeoutA  + ' | ' + CParamSetTimeoutB  + ' | ' + CParamSetTimeoutC + ' <number>');
          WriteLn('< HELP SHOWN');

          LDone := true;
        end;
        if ((LParameter = CParamSetHostA) or (LParameter = CParamSetHostB) or (LParameter = CParamSetHostC)) then
        begin
          LProgress := peSetHost;

          LDone := true;
        end;
        if ((LParameter = CParamSetPortA) or (LParameter = CParamSetPortB) or (LParameter = CParamSetPortC)) then
        begin
          LProgress := peSetPort;

          LDone := true;
        end;
        if ((LParameter = CParamSetTimeoutA) or (LParameter = CParamSetTimeoutB) or (LParameter = CParamSetTimeoutC)) then
        begin
          LProgress := peSetTimeout;

          LDone := true;
        end;

        if not(LDone) then
          WriteLn('< UNKNOWN PARAMETER');
      end
      else
        AShutdown := true;
    end;

    WriteLn('');
  end;
end;

begin
  WriteLn('PNPclient 0.3b');
  WriteLn('(coltishWARE)');
  WriteLn('');
  WriteLn('< STARTING EXECUTION');

  VIdPNPClient := TIdPNPClient.Create(nil);
  try
    VIdPNPClient.ReadTimeout := 1000;

    ExecuteParameters(VExit);
    if not(VExit) then
    begin
      WriteLn('< EXECUTION STARTED');

      VExit := false;
      while not(VExit) do
      begin
        WriteLn('');
        Write('> ');
        ReadLn(VLine);
        ExecuteCommand(VLine, VExit);
      end;
    end;
  finally
    try
      if VIdPNPClient.Connected then
        VIdPNPClient.Disconnect;
    except
    end;

    VIdPNPClient.Free;
    VIdPNPClient := nil;
  end;

  WriteLn('< EXECUTION STOPPED');
  WriteLn('');
  Write('(Press ENTER to Proceed.)');
  ReadLn(VLine);
end.
