unit pnpclss;

interface

uses
  SysUtils,
  Classes,
  IdTCPServer,
  IdTCPClient,
  PNPshr;

type
  TIdPNPClient     = class;
  TIdPNPPeerThread = class;
  TIdPNPServer     = class;

  TIdPNPClient = class(TidTCPClient)
  public
    constructor Create(AOwner : TComponent); override;
  end;

  TIdPNPPeerThread = class(TIdPeerThread)
  protected
    procedure Execute; override;
  public
    constructor Create(ACreateSuspended : Boolean); override;
  end;

  TIdPNPServer = class(TIdTCPServer)
  public
    constructor Create(AOwner : TComponent); override;
  end;

implementation

{ TIdPNPClient }

constructor TIdPNPClient.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);

  Self.Host := '127.0.0.1';
  Self.Port := CPNPDefaultPort;
end;

{ TIdPNPPeerThread }

procedure TIdPNPPeerThread.Execute;
var
  LCommand    : String;
  LDisconnect : Boolean;
  LIndex      : LongInt;
  LResponse   : TStringArray;
begin
  LDisconnect := false;
  SetLength(LResponse, 0);

  while Self.Connection.Connected do
  begin
    LCommand := Self.Connection.ReadLn;
    if (Length(LCommand) > 0) then
    begin
      LResponse := HandleRequest(LCommand, LDisconnect);
      if (Length(LResponse) > 0) then
      begin
        for LIndex := 0 to Pred(Length(LResponse)) do
          Self.Connection.WriteLn(LResponse[LIndex]);
      end;

      if LDisconnect then
        Self.Connection.Disconnect;
    end;
  end;
end;

constructor TIdPNPPeerThread.Create(ACreateSuspended : Boolean);
begin
  inherited Create(ACreateSuspended);

  Self.FreeOnTerminate := true;
end;

{ TIdPNPServer }

constructor TIdPNPServer.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);

  Self.DefaultPort := CPNPDefaultPort;
  Self.ThreadClass := TIdPNPPeerThread;
end;

end.
