program pnpsrvr;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Classes,
  pnpshr  in '../pnpshare/pnpshr.pas',
  pnpclss in '../pnpshare/pnpclss.pas';

var
  VIdPNPServer : TIdPNPServer;
  VIndex       : LongInt;
  VLine        : String;
  VShutdown    : Boolean;
  VThreadList  : TList;

const
  CHelpA = 'HELP';
  CHelpB = 'H';

  CListAddA = 'LISTADD';
  CListAddB = 'LADD';
  CListAddC = 'LA';

  CListClearA = 'LISTCLEAR';
  CListClearB = 'LCLEAR';
  CListClearC = 'LC';

  CListLoadA = 'LISTLOAD';
  CListLoadB = 'LLOAD';
  CListLoadC = 'LL';

  CListSaveA = 'LISTSAVE';
  CListSaveB = 'LSAVE';
  CListSaveC = 'LS';

  CParamHelpA = '-HELP';
  CParamHelpB = '-H';
  CParamHelpC = '-?';

  CParamListLoadA = '-LISTLOAD';
  CParamListLoadB = '-LLOAD';
  CParamListLoadC = '-LL';

  CParamSetPortA = '-SETPORT';
  CParamSetPortB = '-SPORT';
  CParamSetPortC = '-SP';

  CShutDownA = 'SHUTDOWN';
  CShutDownB = 'SDOWN';
  CShutDownC = 'SD';

  CParameterDivider = ' ';

procedure ExecuteCommand(ACommand : String; out AShutdown : Boolean);
var
  LCommand    : String;
  LDone       : Boolean;
  LFileName   : String;
  LParameter  : String;
  LPortName   : String;
  LPortNumber : String;
begin
  ACommand  := AnsiUpperCase(Trim(ACommand));
  AShutdown := false;

  if (Pos(CParameterDivider, ACommand) > 0) then
  begin
    LCommand   := Copy(ACommand, 1, Pred(Pos(CParameterDivider, ACommand)));
    LParameter := Copy(ACommand, Succ(Pos(CParameterDivider, ACommand)), Length(ACommand) - Pos(CParameterDivider, ACommand));
  end
  else
  begin
    LCommand   := ACommand;
    LParameter := '';
  end;

  LDone  := false;
  if ((LCommand = CHelpA) or (LCommand = CHelpB)) then
  begin
    WriteLn('< SHOWING HELP');
    WriteLn('ADD PORTNAME TO LIST: ' + CListAddA   + ' | ' + CListAddB   + ' | ' + CListAddC   + ' <name> <number>');
    WriteLn('CLEAR PORTNAME-LIST : ' + CListClearA + ' | ' + CListClearB + ' | ' + CListClearC);
    WriteLn('LOAD PORTNAME-LIST  : ' + CListLoadA  + ' | ' + CListLoadB  + ' | ' + CListLoadC  + ' <filename>');
    WriteLn('SAVE PORTNAME-LIST  : ' + CListSaveA  + ' | ' + CListSaveB  + ' | ' + CListSaveC  + ' <filename>');
    WriteLn('SHOW HELP           : ' + CHelpA      + ' | ' + CHelpB);
    WriteLn('SHUTDOWN            : ' + CShutdownA  + ' | ' + CShutdownB  + ' | ' + CShutdownC);
    WriteLn('< HELP SHOWN');

    LDone := true;
  end;
  if ((LCommand = CListAddA) or (LCommand = CListAddB) or (LCommand = CListAddC)) then
  begin
    if (Pos(CParameterDivider, LParameter) > 0) then
    begin
      LPortName   := Copy(LParameter, 1, Pred(Pos(CParameterDivider, LParameter)));
      LPortNumber := Copy(LParameter, Succ(Pos(CParameterDivider, LParameter)), Length(LParameter) - Pos(CParameterDivider, LParameter));
    end
    else
    begin
      LPortName   := LParameter;
      LPortNumber := '';
    end;

    WriteLn('< ADDING PORTNAME');
    if AddToPortNameList(LPortName, LPortNumber) then
      WriteLn('< PORTNAME ADDED')
    else
      WriteLn('< ERROR: PORTNAME NOT ADDED SUCCESSFULLY');

    LDone := true;
  end;
  if ((LCommand = CListClearA) or (LCommand = CListClearB) or (LCommand = CListClearC)) then
  begin
    WriteLn('< CLEARING PORTNAME-LIST');
    if ClearPortNameList then
      WriteLn('< PORTNAME-LIST CLEARED')
    else
      WriteLn('< ERROR: PORTNAME-LIST NOT CLEARED SUCCESSFULLY');

    LDone := true;
  end;
  if ((LCommand = CListLoadA) or (LCommand = CListLoadB) or (LCommand = CListLoadC)) then
  begin
    LFileName := LParameter;

    WriteLn('< LOADING PORTNAME-LIST');
    if LoadPortNameList(LFileName) then
      WriteLn('< PORTNAME-LIST LOADED')
    else
      WriteLn('< ERROR: PORTNAME-LIST NOT LOADED SUCCESSFULLY');

    LDone := true;
  end;
  if ((LCommand = CListSaveA) or (LCommand = CListSaveB) or (LCommand = CListSaveC)) then
  begin
    LFileName := LParameter;

    WriteLn('< SAVING PORTNAME-LIST');
    if SavePortNameList(LFileName) then
      WriteLn('< PORTNAME-LIST SAVED')
    else
      WriteLn('< ERROR: PORTNAME-LIST NOT SAVED SUCCESSFULLY');

    LDone := true;
  end;
  if ((LCommand = CShutdownA) or (LCommand = CShutdownB) or (LCommand = CShutdownC)) then
  begin
    AShutdown := true;

    WriteLn('< STOPPING EXECUTION');

    LDone := true;
  end;

  if not(LDone) then
    WriteLn('< UNKNOWN COMMAND');
end;

procedure ExecuteParameters(out AShutdown : Boolean);
type
  TParameterEnum = (peNone, peListLoad, peSetPort);
var
  LDone       : Boolean;
  LIndex      : LongInt;
  LParameter  : String;
  LPortNumber : LongInt;
  LProgress   : TParameterEnum;
begin
  AShutdown := false;

  LProgress := peNone;
  for LIndex := 1 to ParamCount do
  begin
    LParameter := AnsiUpperCase(Trim(ParamStr(LIndex)));

    case LProgress of
      peListLoad :
      begin
        WriteLn('< LOADING PORTNAME-LIST');
        if LoadPortNameList(LParameter) then
          WriteLn('< PORTNAME-LIST LOADED')
        else
          WriteLn('< ERROR: PORTNAME-LIST NOT LOADED SUCCESSFULLY');

        LProgress := peNone;
      end;

      peSetPort :
      begin
        WriteLn('< SETTING PORT');

        LPortNumber := StrToIntDef(LParameter, 0);
        if (LPortNumber > 0) then
        begin
          try
            VIdPNPServer.DefaultPort := LPortNumber;

            WriteLn('< PORT SET');
          except
            WriteLn('< ERROR: PORT NOT SET SUCCESSFULLY');
          end;
        end
        else
          WriteLn('< ERROR: PORT NOT SET SUCCESSFULLY');


        LProgress := peNone;
      end;
    else
      if (LProgress = peNone) then
      begin
        LDone := false;

        if ((LParameter = CParamHelpA) or (LParameter = CParamHelpB) or (LParameter = CParamHelpC)) then
        begin
          AShutdown := true;

          WriteLn('< SHOWING HELP');
          WriteLn('LOAD PORTNAME-LIST: ' + CParamListLoadA + ' | ' + CParamListLoadB + ' | ' + CParamListLoadC + ' <filename>');
          WriteLn('SHOW HELP         : ' + CParamHelpA     + ' | ' + CParamHelpB     + ' | ' + CParamHelpC);
          WriteLn('SET PORT          : ' + CParamSetPortA  + ' | ' + CParamSetPortB  + ' | ' + CParamSetPortC  + ' <number>');
          WriteLn('< HELP SHOWN');

          LDone := true;
        end;
        if ((LParameter = CParamListLoadA) or (LParameter = CParamListLoadB) or (LParameter = CParamListLoadC)) then
        begin
          LProgress := peListLoad;

          LDone := true;
        end;
        if ((LParameter = CParamSetPortA) or (LParameter = CParamSetPortB) or (LParameter = CParamSetPortC)) then
        begin
          LProgress := peSetPort;

          LDone := true;
        end;

        if not(LDone) then
          WriteLn('< UNKNOWN PARAMETER');
      end
      else
        AShutdown := true;
    end;

    WriteLn('');
  end;
end;

begin
  WriteLn('PNPserver 0.3b');
  WriteLn('(coltishWARE)');
  WriteLn('');
  WriteLn('< STARTING EXECUTION');

  VIdPNPServer := TIdPNPServer.Create(nil);
  try
    ClearPortNameList;
    ExecuteParameters(VShutdown);
    if not(VShutdown) then
    begin
      VIdPNPServer.Active := true;

      WriteLn('< EXECUTION STARTED');

      VShutdown := false;
      while not(VShutdown) do
      begin
        WriteLn('');
        Write('> ');
        ReadLn(VLine);
        ExecuteCommand(VLine, VShutdown);
      end;

      VThreadList := VIdPNPServer.Threads.LockList;
      try
        for VIndex := 0 to Pred(VThreadList.Count) do
        begin
          try
            TIdPNPPeerThread(VThreadList[VIndex]).Connection.Disconnect;
          except
          end;
        end;
      finally
        VIdPNPServer.Threads.Clear;
        VIdPNPServer.Threads.UnlockList;
      end;

      VIdPNPServer.Active := false;
    end;
  finally
    VIdPNPServer.Free;
    VIdPNPServer := nil;
  end;

  WriteLn('< EXECUTION STOPPED');
  WriteLn('');
  Write('(Press ENTER to Proceed.)');
  ReadLn(VLine);
end.
