unit Base64U;

// Please, don't delete this comment. \\
(*
  Copyright Owner: coltishWARE (Kevin Niehage)
  Copyright Year : 2007

  Unit   : Base64U.pas (platform independant)
  Version: 0.1a1

  Contact E-Mail            : development@coltishWARE.com
  Contact Mobile            : +49-(0)-177-69-4321-8
  Contact Telephone Berlin  : +49-(0)-30-69-20-96-78
  Contact Telephone Cottbus : +49-(0)-355-28925-88-17-55
  Contact Telephone Mannheim: +49-(0)-621-483-4664
*)
// Please, don't delete this comment. \\

(*
  Description:

  This unit implements Base64 encoding.
*)

(*
  Change Log:

  [Version 0.1a1] (12.06.2007: initial release)
*)

interface

function EncBase64Decode(const AText : String) : String;
function EncBase64Encode(const AText : String) : String;

implementation

const
  CEncBase64FillChar                                  = '=';
  CEncBase64RatioAfter                                = $04;
  CEncBase64RatioBefore                               = $03;
  CEncBase64ShiftMasks    : array [00..07] of LongInt = (-$00, +$02, -$04, +$04, -$02, +$06, -$00, +$00);
  CEncBase64ValueChars    : array [00..63] of Char    = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
  CEncBase64ValueCharsSet : set            of Char    = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
                                                         'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
                                                         'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
                                                         'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
                                                         '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/'];

function EncGetCharIndex(const AChar : Char) : LongInt; forward;
function EncShift(const AByte : Byte; const AShift : Byte; const AShiftRight : Boolean) : Byte; forward;

function EncBoolToInt(const ABoolean : Boolean) : LongInt; forward;
function EncIntToBool(const ALongInt : LongInt) : Boolean; forward;

function EncGetCharIndex(const AChar : Char) : LongInt;
var
  LIndex : LongInt;
begin
  Result := - 1;
  if (AChar in CEncBase64ValueCharsSet) then
  begin
    for LIndex := High(CEncBase64ValueChars) downto Low(CEncBase64ValueChars) do
    begin
      if (AChar = CEncBase64ValueChars[LIndex]) then
      begin
        Result := LIndex;

        Break;
      end;
    end;
  end
  else
  begin
    if (AChar = CEncBase64FillChar) then
      Result := Succ(High(CEncBase64ValueChars));
  end;
end;

function EncShift(const AByte : Byte; const AShift : Byte; const AShiftRight : Boolean) : Byte;
begin
  if (AShift <> $00) then
  begin
    if AShiftRight then
      Result := Byte(AByte shr AShift)
    else
      Result := Byte(AByte shl AShift);
  end
  else
    Result := AByte;
end;

function EncBoolToInt(const ABoolean : Boolean) : LongInt;
begin
  Result := 0;
  if ABoolean then
    Inc(Result);
end;

function EncIntToBool(const ALongInt : LongInt) : Boolean;
begin
  Result := (ALongInt <> 0);
end;

function EncBase64Decode(const AText : String) : String;
const
  CBitMasks : array [01..06] of Byte = ($FC, $03, $F0, $0F, $C0, $3F);
var
  LBreak   : Boolean;
  LBytes   : array [1..CEncBase64RatioAfter] of Byte;
  LCounter : LongInt;
  LCurrent : Byte;
  LDone    : Boolean;
  LIndexA  : LongInt;
  LIndexB  : LongInt;
  LIndexC  : LongInt;
  LValue   : LongInt;
begin
  SetLength(Result, ((Length(AText) div CEncBase64RatioAfter) + EncBoolToInt((Length(AText) mod CEncBase64RatioAfter) <> 0)) * CEncBase64RatioBefore);

  LIndexA := 0;
  LIndexB := 0;
  while (LIndexA < Length(AText)) do
  begin
    LCounter := 0;
    for LIndexC := Low(LBytes) to High(LBytes) do
    begin
      LBytes[LIndexC] := $00;

      if (LIndexA < Length(AText)) then
      begin
        LDone := false;
        while (not(LDone) and (LIndexA < Length(AText))) do
        begin
          Inc(LIndexA);
          LValue := EncGetCharIndex(AText[LIndexA]);
          if ((LValue >= Low(CEncBase64ValueChars)) and (LValue <= High(CEncBase64ValueChars))) then
          begin
            LBytes[LIndexC] := LValue;

            LCounter := LIndexC;
            LDone    := true;
          end;
        end;
      end;
    end;

    if (LCounter > 0) then
    begin
      for LIndexC := 1 to CEncBase64RatioBefore do
      begin
        LBreak := true;

        if (LIndexB < Length(Result)) then
        begin
          LCurrent := $00;

          if (LCounter >= Succ(LIndexC)) then
          begin
            LBreak   := false;
            LCurrent := (EncShift(LBytes[    (LIndexC)], Abs(CEncBase64ShiftMasks[Pred(LIndexC * 2)]), (CEncBase64ShiftMasks[Pred(LIndexC * 2)] < 0)) and CBitMasks[Pred(LIndexC * 2)]) or
                        (EncShift(LBytes[Succ(LIndexC)], Abs(CEncBase64ShiftMasks[    (LIndexC * 2)]), (CEncBase64ShiftMasks[    (LIndexC * 2)] < 0)) and CBitMasks[    (LIndexC * 2)]);
          end;

          if not(LBreak) then
          begin
            Inc(LIndexB);
            Result[LIndexB] := Char(LCurrent);
          end;
        end;

        if LBreak then
          Break;
      end;
    end;
  end;

  SetLength(Result, LIndexB);
end;

function EncBase64Encode(const AText : String) : String;
const
  CBitMasks : array [00..07] of Byte = ($00, $3F, $30, $0F, $3C, $03, $3F, $00);
var
  LBreak   : Boolean;
  LBytes   : array [00..Succ(CEncBase64RatioBefore)] of Byte;
  LCounter : LongInt;
  LCurrent : Byte;
  LIndexA  : LongInt;
  LIndexB  : LongInt;
  LIndexC  : LongInt;
begin
  SetLength(Result, (((Length(AText) div CEncBase64RatioBefore) + EncBoolToInt((Length(AText) mod CEncBase64RatioBefore) <> 0)) * CEncBase64RatioAfter));

  LIndexA := 0;
  LIndexB := 0;
  while (LIndexA < Length(AText)) do
  begin
    LCounter := 0;
    for LIndexC := Low(LBytes) to High(LBytes) do
    begin
      LBytes[LIndexC] := $00;

      if ((LIndexC > Low(LBytes)) and (LIndexC < High(LBytes))) then
      begin
        if (LIndexA < Length(AText)) then
        begin
          Inc(LIndexA);
          LBytes[LIndexC] := Ord(AText[LIndexA]);

          LCounter := LIndexC;
        end
        else
          Break;
      end;
    end;

    if (LCounter > 0) then
    begin
      for LIndexC := 0 to Pred(CEncBase64RatioAfter) do
      begin
        LBreak := true;

        if (LIndexB < Length(Result)) then
        begin
          if (LCounter >= Succ(LIndexC)) then
          begin
            LBreak   := false;
            LCurrent := (EncShift(LBytes[    (LIndexC)], Abs(CEncBase64ShiftMasks[    (LIndexC * 2)]), (CEncBase64ShiftMasks[    (LIndexC * 2)] > 0)) and CBitMasks[    (LIndexC * 2)]) or
                        (EncShift(LBytes[Succ(LIndexC)], Abs(CEncBase64ShiftMasks[Succ(LIndexC * 2)]), (CEncBase64ShiftMasks[Succ(LIndexC * 2)] > 0)) and CBitMasks[Succ(LIndexC * 2)]);
          end
          else
            LCurrent := (EncShift(LBytes[(LIndexC)], Abs(CEncBase64ShiftMasks[(LIndexC * 2)]), (CEncBase64ShiftMasks[(LIndexC * 2)] > 0)) and CBitMasks[(LIndexC * 2)]);

          Inc(LIndexB);
          Result[LIndexB] := CEncBase64ValueChars[LCurrent];
        end;

        if LBreak then
          Break;
      end;
    end;
  end;

  for LIndexA := Succ(LIndexB) to Length(Result) do
    Result[LIndexA] := CEncBase64FillChar;
end;

end.
